/*Data Access Library. v. 1.0b.
 *Visual interface to code generator.
 *AUTHOR:  Alexander Jaremenko <jarem@isb.gomel.by>
 *RELEASE DATE:  02/01/1999
 */

package JProjects.wizzard;

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.sql.*;
import java.util.*;
import JProjects.eab.data.wizzard.*;
import JProjects.beans.*;

/**Visual interface of Data Access Builder.
 *@author Alexander Jaremenko  <address><a href="mailto:jarem@isb.gomel.by">&lt jarem@isb.gomel.by &gt</a></address>
*/
public class Database extends Frame implements ActionListener,TextListener,ItemListener {
  private transient MenuItem _smi,_nmi;
  private transient Connection _con = null;
  private DBMap _map;
  private transient String _ynAction;
  private transient String _mapFile = null;

  //Dynamic GUI
  private transient Panel _center = null;
  private transient GridBagLayout _lm;
  private transient int _curPan;
  private transient List _tbls,_cols,_selCols;
  private transient TextField _tblAlias,_linkString;
  private transient TextField _colType,_fldName;
  private transient Checkbox _isPK,_isNull;
  private transient Label _tblName;
  private transient TextField _pckgName,_clBaseName,_baseDir;
  private transient Checkbox _isRO;
  
  private static final String _fs = System.getProperty("file.separator");

    //DBMap.dirty flag's values
  private static final int CLEAN = 0;
  private static final int NEW = 1;
  private static final int DIRTY = 2;

  public Database() {
    super("Database to Java Classes Mapping");
    init();
    setSize(600,400);
    Dimension d = Toolkit.getDefaultToolkit().getScreenSize();
    setLocation((d.width-600)/2,(d.height-400)/2);
  }

  public void actionPerformed(ActionEvent ev) {
    String com = ev.getActionCommand();
    if (com.equals("yes") || com.equals("no") || com.equals("cancel"))
      _ynAction = com;
    else if (com.equals("NEW"))
      doNewMap();
    else if (com.equals("OPEN"))
      doOpenMap();
    else if (com.equals("SAVE"))
      saveMap();
    else if (com.equals("CONNECT"))
      doConnect();
    else if (com.equals("COLPAN")) {
      CardLayout lm = (CardLayout)_center.getLayout();
      if (_curPan == 3)
	lm.previous(_center);
      else if (_curPan == 1)
	lm.next(_center);
      _curPan = 2;
    } else if (com.equals("CLSPAN")) {
      CardLayout lm = (CardLayout)_center.getLayout();
      if (_curPan == 1)
	lm.previous(_center);
      else if (_curPan == 2)
	lm.next(_center);
      _curPan = 3;
    } else if (com.equals("TBLPAN")) {
      CardLayout lm = (CardLayout)_center.getLayout();
      if (_curPan == 2)
	lm.previous(_center);
      else if (_curPan == 3)
	lm.next(_center);
      _curPan = 1;
    } else if (com.equals("TOSEL")) {
      String[] selCols = _cols.getSelectedItems();
      for(int i=0;i<selCols.length;i++) {
	_cols.remove(selCols[i]);
	_selCols.addItem(selCols[i]);
	_map.selCol(_tbls.getSelectedItem(),selCols[i]);
      }
    } else if (com.equals("FROMSEL")) {
      int sel = _selCols.getSelectedIndex();
      if (sel>=0) {
	String selCol = _selCols.getItem(sel);
	_selCols.remove(sel);
	_cols.addItem(selCol);
	_map.deselCol(_tbls.getSelectedItem(),selCol);
      }
    } else
      doExit();
  }

    public void textValueChanged(TextEvent ev) {
      TextComponent tc = (TextComponent)ev.getSource();
      if (tc==_tblAlias && _tbls.getSelectedItem()!=null) {
	  if (!_map.getTblAlias(_tbls.getSelectedItem()).equals(_tblAlias.getText()))
	      _map.setTblAlias(_tbls.getSelectedItem(),_tblAlias.getText());
      }
      else if (tc==_fldName && _tbls.getSelectedItem()!=null && _selCols.getSelectedItem()!=null) {
	  if (!_map.getFieldName(_tbls.getSelectedItem(),_selCols.getSelectedItem()).equals(_fldName.getText())) 
	      _map.setFieldName(_tbls.getSelectedItem(),_selCols.getSelectedItem(),_fldName.getText());
      }
  }


  public void itemStateChanged(ItemEvent ev) {
    boolean isSel = ev.getStateChange()==ev.SELECTED;
    String tN = _tbls.getSelectedItem();
    if (ev.getItemSelectable()==_isPK && _selCols.getSelectedItem()!=null)
      _map.setPK(tN,_selCols.getSelectedItem(),isSel);
    else if (ev.getItemSelectable()==_isRO)
      _map.setRO(isSel);
    else if (ev.getItemSelectable()==_tbls&&isSel) {
	String ta = _map.getTblAlias(tN);
      _tblAlias.setText((ta==null?"":ta));
      _cols.removeAll();
      _selCols.removeAll();
      Vector cols = _map.getUnsTblCols(tN);
      for (int i=0;i<cols.size();i++)
	_cols.addItem((String)cols.elementAt(i));
      cols = _map.getSelTblCols(tN);
      for (int i=0;i<cols.size();i++)
	_selCols.addItem((String)cols.elementAt(i));
      _colType.setText("");
      _fldName.setText("");
      _tblName.setText(tN);
    } else if (ev.getItemSelectable()==_selCols&&isSel) {
      _colType.setText(_map.getDTString(tN,_selCols.getSelectedItem()));
      _fldName.setText(_map.getFieldName(tN,_selCols.getSelectedItem()));
      _isPK.setState(_map.isPK(tN,_selCols.getSelectedItem()));
      _isNull.setState(_map.isNull(tN,_selCols.getSelectedItem()));
    }
  }

    public static void main(String[] args) {
	new Database().show();
    }

  private void doNewMap() {
      if (saveMap() ) {
	  new MapWizzard(this).show();
	  if (_map.dirty==NEW)
	      initGUI();
      }
  }
  
  private void doConnect() {
      if (  saveMap() ) {
	  new ConnectionPanel(this).show();
	  if (_con != null)
	      _nmi.setEnabled(true);
      }
  }


  private void doExit() {
    if (!saveMap() )
	return;
    dispose();
  }

  private void doOpenMap() {
      if (saveMap()) {
	  FileDialog fd = new FileDialog(this,"Open Map",FileDialog.LOAD);
	  fd.show();
	  if (fd.getFile()!=null) {
	      String mapFile = fd.getDirectory()+_fs+fd.getFile();
	      try {
		  FileInputStream fi = new FileInputStream(mapFile);
		  ObjectInputStream ois = new ObjectInputStream(fi);
		  DBMap map=(DBMap)ois.readObject();
		  ois.close();
		  fi.close();
		  if (!verifyMap(map))
		      map.dirty = DIRTY;
		  else
		      map.dirty = CLEAN;
		  _mapFile = mapFile;
		  _map = map;
		  initGUI();
	      } catch (Exception ex) {
		  handleException(ex);
	      }
	  }
      }
  }

    private void handleException(Exception ex) {
	new YesNoDlg(this,"Exception",ex.getMessage(),"OK",null,null).show();
	System.err.println(ex);
	ex.printStackTrace(System.err);
    }

  private boolean saveMap() {
      if (_map == null )
	  return true;
      if (!_map.getSQLLink().equals(_linkString.getText()))
	  _map.setSQLLink(_linkString.getText());
      if (!_map.getPckg().equals(_pckgName.getText()))
	  _map.setPckg(_pckgName.getText());
      if (!_map.getBaseName().equals(_clBaseName.getText())) 
	  _map.setBaseName(_clBaseName.getText());
      if (!_map.getTargetDir().equals(_baseDir.getText())) 
	  _map.setTargetDir(_baseDir.getText());
      if (_isRO.getState() && !_map.isRO() || _map.isRO() && !_isRO.getState())
	  _map.setRO(_isRO.getState());
      if (_map.dirty == CLEAN)
	  return true;

      YesNoDlg yd = new YesNoDlg(this,"Map needs to be saved","Save this mapping?","Yes","No","Cancel");
      yd.addActionListener(this);
      yd.show();
      if (_ynAction.equals("cancel"))
	  return false;
      if (_ynAction.equals("yes")) {
    String mapFile = null;
    int chck = _map.checkConsistency(),fl;
    if (chck != OK) {
      String msg="Incomplete map. Missed:\n";
      if ((fl=chck&LINK_MISS)>0)
	msg+="SQL Link String\n";
      if ((fl=chck&PACK_MISS)>0)
	msg+="Package name\n";
      if ((fl=chck&BASE_MISS)>0)
	msg+="Base name";
      new YesNoDlg(this,"Error",msg,"OK",null,null).show();
      return false;
    }
    if ((fl=_map.dirty&NEW)>0) {
      FileDialog fd = new FileDialog(this,"Save New Map",FileDialog.SAVE);
      fd.show();
      if (fd.getFile()!=null) {
	mapFile = fd.getDirectory()+_fs+fd.getFile();
      }
    } else
      mapFile = _mapFile;
    if (mapFile!=null)
      try {
	DCGenerator gen = new DCGenerator(_map.getAllTbls(),_map.getAliases(),_map.getFullBaseName(),_map.getAllCols(),_map.getAllFlds(),_map.getDTypes(),_map.getAllNull(),_map.getAllPK(),_map.getSQLLink());
	gen.setReadOnly(_map.isRO());
	String tgtDir = (_map.getTargetDir().length()==0?".":_map.getTargetDir());
	gen.generate(tgtDir);
	FileOutputStream fo = new FileOutputStream(mapFile);
	ObjectOutputStream oos = new ObjectOutputStream(fo);
	_map.dirty = CLEAN;
	oos.writeObject(_map);
	oos.flush();
	oos.close();
	fo.close();
	_mapFile = mapFile;
	new YesNoDlg(this,"Information","Classes has been generated.\nMap saved.","OK",null,null).show();
      } catch (IOException ex) {
	handleException(ex);
	return false;
      }
      }
      return true;
  }

  private boolean verifyMap(DBMap map) throws SQLException {
    boolean res = true;
    if (_con != null) {
      DatabaseMetaData md = _con.getMetaData();
      for (Enumeration tblEls=map.getTables();tblEls.hasMoreElements();) {
	String tN = (String)tblEls.nextElement();
	String tblN = (tN.indexOf('.')>0?tN.substring(tN.indexOf('.')+1):tN);
	String schN = (tN.indexOf('.')>0?tN.substring(0,tN.indexOf('.')):"");
	ResultSet rs = md.getTables(null,schN,tblN,new String[] {"TABLE"});
	if (!rs.next()) {
	  YesNoDlg yd = new YesNoDlg(this,"Error","Table "+tN+" is not present in DB: delete it from the map?","Yes","No",null);
	  yd.addActionListener(this);
	  yd.show();
	  if (_ynAction.equals("yes")) {
	    map.delTbl(tN);
	    res = false;
	  }
	} else {
	  // see for columns
	  rs = md.getColumns(null,schN,tblN,"");
	  Hashtable dbCols = new Hashtable();
	  while(rs.next()) 
	    dbCols.put(rs.getString(4),new int[] {rs.getInt(5),rs.getInt(9),rs.getInt(11)});
	  rs.close();
	  Vector vec = map.getCols(tN);
	  for (int i=0;i<vec.size();i++) {
	    String colN = (String)vec.elementAt(i);
	    if (!dbCols.containsKey(colN)) {
	      res = false;
	      new YesNoDlg(this,"Information","There is no column "+colN+" in the "+tN+" ! Deleting ..","OK",null,null).show();
	      map.delCol(tN,colN);
	    }
	    else {
	      int[] dt = (int[])dbCols.get(colN);
	      int[] cDt = map.getColTypeNull(tN,colN);
	      if (dt[0]!=cDt[0] || dt[1]!=cDt[1] || dt[2]!=cDt[2]) {
		res = false;
		new YesNoDlg(this,"Information","Type or nullable property of "+colN+" column of the "+tN+" table had been changed! Updating ..","OK",null,null);
		map.setCol(tN,colN,dt[0],dt[1],dt[2]);
	      }
	    }
	  }
	  //check for added cols
	  if (map.getColCount(tN)<dbCols.size()) {
	    res = false;
	    new YesNoDlg(this,"Information","New columns had been added to the "+tN+" ! Updating ..","OK",null,null);
	    for (Enumeration addCols=dbCols.keys();addCols.hasMoreElements();){
	      String acN = (String)addCols.nextElement();
	      if (!map.hasCol(tN,acN)) {
		int[] dt= (int[])dbCols.get(acN);
		map.addCol(tN,acN,dt[0],dt[1],dt[2]);
	      }
	    }
	  }
	  //check for PK 
	  rs = md.getPrimaryKeys(null,schN,tblN);
	  while(rs.next()) {
	    String pkN = rs.getString(4);
	    if (!map.isPK(tN,pkN)) {
	      res = false;
	      new YesNoDlg(this,"Information","Primary key of the "+tN+" table had been changed! Updating ..","OK",null,null);
	      map.setPK(tN,pkN);
	    }
	  }
	}  
      }
    }
    return res;
  }

  private void init() {
    MenuBar mb = new MenuBar();
    Menu m = new Menu("File");
    MenuItem mi = new MenuItem("Connect to DB..");
    mi.setActionCommand("CONNECT");
    mi.addActionListener(this);
    m.add(mi);
    _nmi = new MenuItem("New map..");
    _nmi.setActionCommand("NEW");
    _nmi.addActionListener(this);
    m.add(_nmi);
    _nmi.setEnabled(false);
    mi = new MenuItem("Open map..");
    mi.setActionCommand("OPEN");
    mi.addActionListener(this);
    m.add(mi);
    _smi = new MenuItem("Save map");
    _smi.setActionCommand("SAVE");
    _smi.addActionListener(this);
    m.add(_smi);
    _smi.setEnabled(false);
    mi = new MenuItem("Exit");
    mi.setActionCommand("EXIT");
    mi.addActionListener(this);
    m.add(mi);
    mb.add(m);
    setMenuBar(mb);
    addWindowListener(new WindowAdapter() {
      public void windowClosing(WindowEvent e) {
	doExit();
      }
    });
  }

  private void initGUI() {
    buildGUI();
    //display data from _map
    _tbls.removeAll();
    for(Enumeration tbls = _map.getTables();tbls.hasMoreElements();)
      _tbls.addItem((String)tbls.nextElement());
    _linkString.setText(_map.getSQLLink());
    if (_map.getTblCount()==1) {
      _linkString.setEnabled(false);
      _isRO.setState(_map.isRO());
      _isRO.setEnabled(true);
    }
    else {
      _linkString.setEnabled(true);
      _isRO.setState(true);
      _isRO.setEnabled(false);
      if (!_map.isRO())
	  _map.setRO(true);
    }
    _cols.removeAll();
    _selCols.removeAll();
    _colType.setText("");
    _fldName.setText("");
    _pckgName.setText(_map.getPckg());
    _clBaseName.setText(_map.getBaseName());
    _baseDir.setText(_map.getTargetDir());
    _tbls.select(0);
    String tN = _tbls.getItem(0);
    _tblName.setText(tN);
    _tblAlias.setText(_map.getTblAlias(tN));
    Vector cols = _map.getUnsTblCols(tN);
    for (int i=0;i<cols.size();i++)
	_cols.addItem((String)cols.elementAt(i));
    cols = _map.getSelTblCols(tN);
    for (int i=0;i<cols.size();i++)
	_selCols.addItem((String)cols.elementAt(i));
  }

  private void buildGUI() {
    if (_center==null) {
      _center = new Panel(new CardLayout());
      _lm=new GridBagLayout();
      _center.add(crTblPan(),"TBLPAN");
      _center.add(crColPan(),"COLPAN");
      _center.add(crClsPan(),"CLASSPAN");
      add("Center",_center);
      Panel north = new Panel();
      Button b = new Button("Tables");
      b.setActionCommand("TBLPAN");
      b.addActionListener(this);
      north.add(b);
      b = new Button("Columns");
      b.setActionCommand("COLPAN");
      b.addActionListener(this);
      north.add(b);
      b = new Button("Classes");
      b.setActionCommand("CLSPAN");
      b.addActionListener(this);
      north.add(b);
      add("North",north);
      _curPan=1;
      _smi.setEnabled(true);
      validate();
    }
  }

  private Panel crTblPan() {
    Panel res = new Panel(_lm);
    Label lab = new Label("Tables");
    addCell(lab,res,1,0);
    lab = new Label("Table's alias");
    addCell(lab,res,2,0);
    _tbls = new List();
    _tbls.addItemListener(this);
    addCell(_tbls,res,3,3);
    _tblAlias=new TextField();
    _tblAlias.addTextListener(this);
    addCell(_tblAlias,res,2,0);
    lab = new Label("SQL Link String");
    addCell(lab,res,2,0);
    _linkString = new TextField();
    addCell(_linkString,res,4,0);
    return res;
  }

  private Panel crColPan() {
    Panel res = new Panel(_lm);
    _tblName = new Label("",Label.CENTER);
    addCell(_tblName,res,2,0);
    Label lab = new Label("Available");
    addCell(lab,res,5,2);
    lab = new Label("Selected");
    addCell(lab,res,2,0);
    _cols = new List();
    _cols.setMultipleMode(true);
    addCell(_cols,res,5,2);
    _selCols = new List();
    _selCols.addItemListener(this);
    addCell(_selCols,res,2,0);
    Button b = new Button(">");
    b.setActionCommand("TOSEL");
    b.addActionListener(this);
    addCell(b,res,5,2);
    b = new Button("<");
    b.setActionCommand("FROMSEL");
    b.addActionListener(this);
    addCell(b,res,2,0);
    lab = new Label("Column's information",Label.CENTER);
    addCell(lab,res,2,0);
    lab = new Label("Type");
    addCell(lab,res,1,0);
    _colType = new TextField();
    _colType.setEditable(false);
    addCell(_colType,res,1,0);
    lab = new Label("Field Name");
    addCell(lab,res,1,0);
    _fldName = new TextField();
    _fldName.addTextListener(this);
    addCell(_fldName,res,2,0);
    _isPK = new Checkbox("Is PK");
    _isPK.addItemListener(this);
    addCell(_isPK,res,5,2);
    _isNull = new Checkbox("Is Nullable");
    _isNull.setEnabled(false);
    addCell(_isNull,res,2,0);
    return res;
  }

  private Panel crClsPan() {
    Panel res = new Panel(_lm);
    Label lab = new Label("Package");
    addCell(lab,res,1,0);
    _pckgName = new TextField();
    addCell(_pckgName,res,2,0);
    lab = new Label("Base Name");
    addCell(lab,res,4,0);
    _clBaseName = new TextField();
    addCell(_clBaseName,res,2,0);
    lab = new Label("Target Directory");
    addCell(lab,res,4,0);
    _baseDir = new TextField();
    addCell(_baseDir,res,2,0);
    _isRO = new Checkbox("Is read only");
    addCell(_isRO,res,2,0);
    return res;
  }

  private void addCell(Component com,Container con,int var,int subv) {
    GridBagLayout lm = (GridBagLayout)con.getLayout();
    GridBagConstraints cs = new GridBagConstraints();
    cs.insets = new Insets(2,2,2,2);
    cs.weightx = 1;
    cs.fill = cs.BOTH;
    switch(var) {
    case 1: //1-st col
      cs.gridwidth = 1;
      break;
    case 2: //last col
      cs.gridwidth = cs.REMAINDER;
      break;
    case 3: // cell has subv lines in height
      cs.weighty = 1;
      cs.gridheight = subv;
      break;
    case 4: // cell occupies space next to previous
      cs.gridwidth = cs.RELATIVE;
      break;
    case 5: //cell has subv cols in width
	cs.gridwidth = subv;
	break;
    }
    lm.setConstraints(com,cs);
    con.add(com);
  }

  //Connect to DB dialog
  private class ConnectionPanel extends Dialog implements ActionListener {
    private TextField _drvTF;
    private TextField _urlTF;
    private TextField _usrTF;
    private TextField _pwdTF;
    
    public ConnectionPanel(Frame par) {
      super(par,"Connecting to Database",true);
      init();
      setSize(400,300);
      Dimension d = Toolkit.getDefaultToolkit().getScreenSize();
      setLocation((d.width-400)/2,(d.height-300)/2);
    }
    
    public void actionPerformed(ActionEvent ev) {
      if (ev.getActionCommand().equals("OK")) {
	if (_urlTF.getText().length()==0) {
	  YesNoDlg yd = new YesNoDlg((Frame)getParent(),"Error","Supply URL to connect to","Yes",null,null);
	  yd.show();
	  return;
	}
	try {
	  if (_drvTF.getText().length()>0)
	    Class.forName(_drvTF.getText());
	  if (_usrTF.getText().length()>0 && _pwdTF.getText().length()>0)
	    _con = DriverManager.getConnection(_urlTF.getText(),_usrTF.getText(),_pwdTF.getText());
	  else
	    _con = DriverManager.getConnection(_urlTF.getText());
	} catch (Exception ex) {
	  System.out.println(ex.getMessage());
	  ex.printStackTrace();
	  YesNoDlg yd = new YesNoDlg((Frame)getParent(),"Exception","Couldn't connect.","Yes",null,null);
	  yd.show();
	  return;
	}
      }
      dispose();
    }

    private void init() {
      GridBagConstraints cs = new GridBagConstraints();
      GridBagLayout lm = new GridBagLayout();
      Insets ins = new Insets(2,2,2,2);
      Panel center = new Panel(lm);
      cs.fill = cs.HORIZONTAL;
      cs.insets = ins;
      cs.weightx=1;
      Label la = new Label("Driver");
      lm.setConstraints(la,cs);
      center.add(la);
      cs.gridwidth=cs.REMAINDER;
      _drvTF = new TextField();
      lm.setConstraints(_drvTF,cs);
      center.add(_drvTF);
      cs.gridwidth=cs.RELATIVE;
      la = new Label("URL");
      lm.setConstraints(la,cs);
      center.add(la);
      cs.gridwidth=cs.REMAINDER;
      _urlTF = new TextField();
      lm.setConstraints(_urlTF,cs);
      center.add(_urlTF);
      cs.gridwidth=cs.RELATIVE;
      la = new Label("User");
      lm.setConstraints(la,cs);
      center.add(la);
      cs.gridwidth=cs.REMAINDER;
      _usrTF = new TextField();
      lm.setConstraints(_usrTF,cs);
      center.add(_usrTF);
      cs.gridwidth=cs.RELATIVE;
      la = new Label("Password");
      lm.setConstraints(la,cs);
      center.add(la);
      cs.gridwidth=cs.REMAINDER;
      _pwdTF=new TextField();
      _pwdTF.setEchoChar('*');
      lm.setConstraints(_pwdTF,cs);
      center.add(_pwdTF);
      add("Center",center);
      Panel south = new Panel();
      Button b = new Button("Connect");
      b.addActionListener(this);
      b.setActionCommand("OK");
      south.add(b);
      b = new Button("Cancel");
      b.addActionListener(this);
      south.add(b);
      add("South",south);
    }

  }

  //Select table(s) dialog.
  private class MapWizzard extends Dialog implements ActionListener {
    private TextField _schemaMsk,_tblMsk;
    private List _tbls;
    private Vector _schemas;

    public MapWizzard(Frame par) {
      super(par,"Database Map Wizzard",true);
      init();
      setSize(400,400);
      Dimension d = Toolkit.getDefaultToolkit().getScreenSize();
      setLocation((d.width-400)/2,(d.height-400)/2);
    }

    public void actionPerformed(ActionEvent ev) {
      String com = ev.getActionCommand();
      if (com.equals("RETRIEVE"))
	doRetrieve();
      else if (com.equals("OK")) {
	int[] si = _tbls.getSelectedIndexes();
	if (si.length>0)
	  doMap();
      }
      else
	dispose();
    }
    
    private void doRetrieve() {
      //retrieve table names
      try {
	DatabaseMetaData md = _con.getMetaData();
	boolean isSchema = _schemaMsk.getText().length()>0;
	ResultSet rs = md.getTables(null,_schemaMsk.getText(),_tblMsk.getText(),new String[] {"TABLE"});
	_tbls.removeAll();
	_schemas.removeAllElements();
	while(rs.next()) {
	  String sch="";
	  if (isSchema) 
	      sch = rs.getString(2);
	  _tbls.addItem(rs.getString(3));
	  _schemas.addElement(sch);
	}
      } catch (SQLException ex) {
	handleException(ex);
      }
    }

    private void doMap() {
      //make _map
      try {
	DatabaseMetaData md = _con.getMetaData();
	int selTbls[] = _tbls.getSelectedIndexes();
	_map = new DBMap();
	for (int i=0;i<selTbls.length;i++) {
	  String schN = (String)_schemas.elementAt(selTbls[i]);
	  String tN = _tbls.getItem(selTbls[i]);
	  String tablN = (schN.length()>0?schN+"."+tN:tN);
	  _map.addTbl(tablN);
	  ResultSet rs = md.getColumns(null,schN,tN,"");
	  while(rs.next()) 
	    _map.addCol(tablN,rs.getString(4),rs.getInt(5),rs.getInt(9),rs.getInt(11));
	  rs = md.getPrimaryKeys(null,schN,tN);
	  while(rs.next())
	    _map.setPK(tablN,rs.getString(4));
	}
	dispose();
      } catch (SQLException ex) {
	handleException(ex);
      }
    }

    private void init() {
      _schemas = new Vector();
      Panel center = new Panel(new GridBagLayout());
      Label lab = new Label("Schema mask");
      addCell(lab,center,1,0);
      _schemaMsk = new TextField();
      addCell(_schemaMsk,center,2,0);
      lab = new Label("Table mask");
      addCell(lab,center,4,0);
      _tblMsk = new TextField();
      addCell(_tblMsk,center,2,0);
      _tbls = new List(10,true);
      //      _tbls.setMultipleMode(true);
      addCell(_tbls,center,2,0);
      add("Center",center);
      Panel south = new Panel();
      Button b = new Button("Show Tables");
      b.setActionCommand("RETRIEVE");
      b.addActionListener(this);
      south.add(b);
      b = new Button("Make Map");
      b.setActionCommand("OK");
      b.addActionListener(this);
      south.add(b);
      b = new Button("Cancel");
      b.setActionCommand("CANCEL");
      b.addActionListener(this);
      south.add(b);
      add("South",south);
    }

  }

    //DBMap.checkConsistency() return codes
    private static final int OK=0;
      //    public static final int ALIAS_MISS=1;
    private static final int LINK_MISS=2;
    private static final int PACK_MISS=4;
    private static final int BASE_MISS=8;

  //DB to Java classes map data container
  private class DBMap implements Serializable {
    private Hashtable _tblDsc;
    private boolean _isRO = false;
    private String _pckg = "";
    private String _baseN = "";
    private String _targetDir = "";
    private String _SQLLink = "";
      public int dirty = NEW;
    
    public DBMap() {
      _tblDsc = new Hashtable();
    }

    //return error code: OK or combination of other error codes.
      public int checkConsistency() {
	  int res = OK;
	  if (_tblDsc.size()>1 && _SQLLink.length()==0)
	      res|=LINK_MISS;
	  if (_pckg.length()==0)
	      res|=PACK_MISS;
	  if (_baseN.length()==0)
	      res|=BASE_MISS;
	  return res;
      }

      //PK is constituted by 1-st table's PK cols. And it must be on head of the list
      public void arrangePK() {
	  if (_tblDsc.size()>0) {
	      Enumeration te = _tblDsc.keys();
	      TblDescr t1 = (TblDescr)_tblDsc.get(te.nextElement());
	      t1.arrangePK();
	  }
      }

    //table level

    public void addTbl(String tN) {
      if (!_tblDsc.containsKey(tN))
	_tblDsc.put(tN,new TblDescr(tN,"t"+(_tblDsc.size()+1)));
    }

    public int getTblCount() {
      return _tblDsc.size();
    }

    public Enumeration getTables() {
      return _tblDsc.keys();
    }

    public void delTbl(String tN) {
      _tblDsc.remove(tN);
    }

    public String getTblAlias(String tN) {
      if (!_tblDsc.containsKey(tN))
	return null;
      return ((TblDescr)_tblDsc.get(tN)).aN;
    }

    public void setTblAlias(String tN,String aN) {
	if (_tblDsc.containsKey(tN)) {
	    ((TblDescr)_tblDsc.get(tN)).aN = aN;
	    dirty = (dirty!=NEW?DIRTY:dirty);
	}
    }

    //column - level

    public void addCol(String tN,String cN,int typ,int sc,int nlFlg) {
	if (_tblDsc.containsKey(tN)) {
	    TblDescr t = (TblDescr)_tblDsc.get(tN);
	    t.cols.addElement(new ColDescr(t,cN,typ,sc,nlFlg));
	}
    }

    public void delCol(String tN,String cN) {
	if (_tblDsc.containsKey(tN)) {
	    TblDescr t = (TblDescr)_tblDsc.get(tN);
	    t.cols.removeElement(new ColDescr(t,cN));
	}
    }

    public void setCol(String tN,String cN,int typ,int sc,int nlFlg) {
      ColDescr cd = findCol(tN,cN);
      if (cd!=null) {
	cd.dt = typ;
	cd.scale = sc;
	cd.nlFlg = nlFlg;
      }
    }

    public void setPK(String tN,String cN) {
      setPK(tN,cN,true);
    }

    public void setPK(String tN,String cN,boolean flg) {
      ColDescr cd = findCol(tN,cN);
      if (cd!=null) 
	cd.isPK = flg;
    }

    public boolean isPK(String tN,String cN) {
      ColDescr cd = findCol(tN,cN);
      return (cd!=null && cd.isPK);
    }

      public boolean isColModi(String tN,String cN) {
	  ColDescr cd = findCol(tN,cN);
	  return cd!=null && cd.isModi;
      }

      public void setColModi(String tN,String cN,boolean flag) {
	  ColDescr cd = findCol(tN,cN);
	  if (cd!=null)
	      cd.isModi = flag;
      }

    public boolean hasCol(String tN,String cN) {
      return findCol(tN,cN)!=null;
    }

    public Vector getCols(String tN) {
      Vector res = new Vector();
      if (_tblDsc.containsKey(tN)) {
	Enumeration els = ((TblDescr)_tblDsc.get(tN)).cols.elements();
	while(els.hasMoreElements())
	  res.addElement(((ColDescr)els.nextElement()).colN);
      }
      return res;
    }

    public int getColCount(String tN) {
      int res=0;
      if (_tblDsc.containsKey(tN)) 
	res = ((TblDescr)_tblDsc.get(tN)).cols.size();
      return res;
    }

    public int[] getColTypeNull(String tN,String cN) {
      ColDescr cd = findCol(tN,cN);
      if (cd!=null)
	return new int[] {cd.dt,cd.scale,cd.nlFlg};
      return null;
    }

    public boolean isNull(String tN,String cN) {
      ColDescr cd = findCol(tN,cN);
      if (cd!=null)
	return cd.isNullable();
      return false;
    }


    //data for generator

    public String getFieldName(String tN,String cN) {
      ColDescr cd = findCol(tN,cN);
      if (cd!=null)
	return cd.fldN;
      return null;
    }

    public void setFieldName(String tN,String cN,String fN) {
      ColDescr cd = findCol(tN,cN);
      if (cd!=null) {
	cd.fldN = fN;
	dirty= (dirty!=NEW?DIRTY:dirty);
      }
    }

    public String getDTString(String tN,String cN) {
      ColDescr cd = findCol(tN,cN);
      if (cd!=null)
	return cd.getDTString();
      return null;
    }

    public String getTargetDir() {
      return _targetDir;
    }

    public void setTargetDir(String dN) {
      _targetDir = dN;
      dirty= (dirty!=NEW?DIRTY:dirty);
    }

    public String getBaseName() {
      return _baseN;
    }

    public void setBaseName(String bN) {
      _baseN = bN;
      dirty= (dirty!=NEW?DIRTY:dirty);
    }

    public String getPckg() {
      return _pckg;
    }

    public void setPckg(String pN) {
      _pckg = pN;
      dirty= (dirty!=NEW?DIRTY:dirty);
    }

    public boolean isRO() {
      return _isRO;
    }

    public void setRO(boolean flg) {
      _isRO=flg;
      dirty= (dirty!=NEW?DIRTY:dirty);
    }

    public void selCol(String tN,String cN) {
      ColDescr cd = findCol(tN,cN);
      if (cd!=null) {
	dirty= (dirty!=NEW?DIRTY:dirty);
	cd.isSel = true;
      }
    }

    public void deselCol(String tN,String cN) {
      ColDescr cd = findCol(tN,cN);
      if (cd!=null) {
	dirty= (dirty!=NEW?DIRTY:dirty);
	cd.isSel = false;
      }
    }

    public Vector getSelTblCols(String tN) {
      return getTblCols(tN,true);
    }

    public Vector getUnsTblCols(String tN) {
      return getTblCols(tN,false);
    }

      public Vector getTblCols(String tN,boolean isSel) {
	  Vector res = new Vector();
	  if (_tblDsc.containsKey(tN)) {
	      Vector cds = ((TblDescr)_tblDsc.get(tN)).cols;
	      for (int i=0;i<cds.size();i++)
		  if (((ColDescr)cds.elementAt(i)).isSel&&isSel|| !isSel&&!((ColDescr)cds.elementAt(i)).isSel)
		      res.addElement(((ColDescr)cds.elementAt(i)).colN);
	  }
	  return res;
      }

    //methods used to invoke DCGenerator's constructor

    public String getSQLLink() {
      return _SQLLink;
    }

    public void setSQLLink(String sqlL) {
      _SQLLink = sqlL;
      dirty= (dirty!=NEW?DIRTY:dirty);
    }

      public String[] getAllTbls() {
	  String[] res = new String[_tblDsc.size()];
	  int i=0;
	  for (Enumeration tbls = _tblDsc.keys();tbls.hasMoreElements();i++)
	      res[i]=(String)tbls.nextElement();
	  return res;
      }

      public String[] getAliases() {
	  String[] res = new String[_tblDsc.size()];
	  int i=0;
	  for (Enumeration tbls = _tblDsc.keys();tbls.hasMoreElements();i++)
	      res[i]=((TblDescr)_tblDsc.get(tbls.nextElement())).aN;
	  return res;
      }

      public String getFullBaseName() {
	  return (_pckg.length()>0?_pckg+"."+_baseN:_baseN);
      }

      public String[] getAllCols() {
	  Vector cols = getAllColDescr();
	  String[] res = new String[cols.size()];

	  for (int j=0;j<res.length;j++) {
	      ColDescr col = (ColDescr)cols.elementAt(j);
	      res[j]=col.tbl.aN+"."+col.colN;
	  }
	  return res;
      }

      public String[] getAllFlds() {
	  Vector cols = getAllColDescr();
	  String[] res = new String[cols.size()];
	  
	  for (int i=0;i<res.length;i++) {
	      ColDescr col = (ColDescr)cols.elementAt(i);
	      res[i] = (col.colN.equals(col.fldN)?col.tbl.aN+col.fldN:col.fldN);
	  }
	  return res;
      }

      public int[][] getDTypes() {
	  Vector cols = getAllColDescr();
	  int[][] res = new int[cols.size()][2];
	  
	  for (int i=0;i<res.length;i++) {
	      ColDescr col = (ColDescr)cols.elementAt(i);
	      res[i][0] = col.dt;
	      res[i][1] = col.scale;
	  }
	  return res;
      }

    public boolean[] getAllNull() {
	Vector cols = getAllColDescr();
	boolean[] res = new boolean[cols.size()];

	for(int i=0; i<res.length; i++)
	    res[i]= ((ColDescr)cols.elementAt(i)).isNullable();
	return res;
    }

    public boolean[] getAllPK() {
	Enumeration tbls = _tblDsc.keys();
	TblDescr t1 = (TblDescr)_tblDsc.get(tbls.nextElement());
	Vector cols = t1.cols;
	int pkCnt=0,i;
	for(pkCnt=0;pkCnt<cols.size()&&((ColDescr)cols.elementAt(pkCnt)).isPK;pkCnt++) ;
	boolean[] res = new boolean[pkCnt];
	for (i=0;i<pkCnt;i++)
	    res[i]=((ColDescr)cols.elementAt(i)).isModi;
	return res;
    }

    //gets all selected cols (ColDescr)
      private Vector getAllColDescr() {
	  int colCnt=0;
	  int i=0;
	  String[] tbls = getAllTbls();
	  Vector rv = new Vector();

	  for (i=0,colCnt=0;i<tbls.length;i++) {
	      TblDescr t = (TblDescr)_tblDsc.get(tbls[i]);
	      for (int j=0;j<t.cols.size();j++) {
		  ColDescr col = (ColDescr)t.cols.elementAt(j);
		  if (col.isSel)
		      rv.addElement(col);
	      }
	  }
	  return rv;
      }

    private ColDescr findCol(String tN,String cN) {
      if (_tblDsc.containsKey(tN)) {
	TblDescr td = (TblDescr)_tblDsc.get(tN);
	int pos = td.cols.indexOf(new ColDescr(td,cN));
	if (pos>=0)
	  return (ColDescr)td.cols.elementAt(pos);
      }
      return null;
    }

    //table descriptor
    private class TblDescr implements Serializable {
      public String tN;
      public String aN;
      public Vector cols;
      
      public TblDescr(String tN,String aN) {
	this.tN = tN;
	this.aN = aN;
	cols = new Vector();
      }

	public void arrangePK() {
	    Vector ncols = new Vector();
	    int i=0;
	    for (;i<cols.size();i++)
		if (((ColDescr)cols.elementAt(i)).isPK)
		    ncols.addElement(cols.elementAt(i));
	    for (;i<cols.size();i++)
		if (!((ColDescr)cols.elementAt(i)).isPK)
		    ncols.addElement(cols.elementAt(i));
	    cols = ncols;
	}

    }

    //column descriptor
    private class ColDescr implements Serializable {
      public String colN;
      public String fldN;
      public int dt;
      public int scale;
      public int nlFlg;
      public boolean isPK;
      public boolean isSel = false;
	public TblDescr tbl;
	public boolean isModi = true;

      public ColDescr(TblDescr t,String cN) {
	this(t,cN,Types.CHAR,0,DatabaseMetaData.columnNullable);
      }

      public ColDescr(TblDescr t,String cN,int dataT,int sc,int nF) {
	  tbl = t;
	fldN=colN=cN;
	dt=dataT;
	scale = sc;
	nlFlg = nF;
	isPK=false;
      }

      public boolean isNullable() {
	return nlFlg!=DatabaseMetaData.columnNoNulls;
      }

      public boolean equals(Object o) {
	if (!(o instanceof ColDescr))
	  return false;
	return colN.equals(((ColDescr)o).colN);
      }

      public String getDTString() {
	return JDBCTypes.getSQLType(dt);
      }

    }

  }

}
