import java.sql.*;
import java.util.*;
import JProjects.eab.data.*;
import java.io.Serializable;


class SampleJoinDataIdManager extends DAManager {

  //----------------------------------------------------------------
  // Constructors
  //----------------------------------------------------------------
  public SampleJoinDataIdManager() {}
  public SampleJoinDataIdManager(DatastoreJDBC aDatastore) {
    super(aDatastore);
  }
  public DatastoreJDBC getCurrentDatastore() {
    if (getObjectsDatastore() != null) return getObjectsDatastore();
    else if (SampleJoin.getDefaultDatastore() != null) return SampleJoin.getDefaultDatastore();
    else return DatastoreJDBC.getApplicationDatastore();
  }
//----------------------------------------------------------------
// Open cursor with parameters
//----------------------------------------------------------------
  public void prepareOpen(String sqlSuffix,Object[] vals,int[] types) throws DAException {
    if (sqlSuffix==null|| vals==null|| types==null)
      throw new IllegalArgumentException();
    if (vals.length != types.length)
      throw new IllegalArgumentException();
    if ( !_onBackground() && isAsynchronous() ) {
      Object[] params = {sqlSuffix,vals,types};
      _putOnBackgroundThread( "prepareOpen(..)", params );
      return ;
    }
    try{
      if (getCurrentDatastore() == null) throw new DAException(DAResource.NO_CONNECT_EXIST);
      if (isOpen()) close();
      _stmt = getCurrentDatastore().getConnection().prepareStatement(getSQLBaseQuery() + sqlSuffix);
      _setStmtOptions();
      for (int i=0;i<types.length;i++)
        _stmt.setObject(i+1,vals[i],types[i]);
      _rs = _stmt.executeQuery();
      _map = new SampleJoinDataIdMap();
      _setOpen(true);
    } catch(SQLException exc) {
      throw new DAException(DAResource.PREP_OPEN_FAILED, exc);
    } finally { _setBusy(false); }
    _managerSupport.firePrepareOpenComplete("prepareOpen(..)");
  }
//----------------------------------------------------------------
// Open cursor
//----------------------------------------------------------------
  public void open(String sqlSuffix) throws DAException {
    if ( !_onBackground() && isAsynchronous() ) {
      Object[] params = {sqlSuffix};
      _putOnBackgroundThread( "open(String)", params );
      return ;
    }
    try {
    if (getCurrentDatastore() == null) throw new DAException(DAResource.NO_CONNECT_EXIST);
      if (isOpen()) close();
      if (sqlSuffix == null) sqlSuffix = "";
      _stmt = getCurrentDatastore().getConnection().prepareStatement(getSQLBaseQuery() + sqlSuffix);
      _setStmtOptions();
      _rs = _stmt.executeQuery();
      _map = new SampleJoinDataIdMap();
      _setOpen(true);
    } catch(SQLException exc) {
      throw new DAException(DAResource.OPEN_FAILED, exc);
    } finally {
      _setBusy(false);
    }
    _managerSupport.fireOpenComplete("open(String)");
  }
  //----------------------------------------------------------------
  // Select rows and fill internal sequence.
  //----------------------------------------------------------------
  public void select(String sqlSuffix) throws DAException {
    if ( !_onBackground() && isAsynchronous() ) {
      Object[] params = {sqlSuffix};
      _putOnBackgroundThread( "select(String)", params );
      return ;
    }
    try {
    if (getCurrentDatastore() == null) throw new DAException(DAResource.NO_CONNECT_EXIST);
      if (isOpen()) close();
      if (sqlSuffix == null) sqlSuffix = "";
      _stmt = getCurrentDatastore().getConnection().prepareStatement(getSQLBaseQuery() + sqlSuffix);
      _setStmtOptions();
      _rs = _stmt.executeQuery();
      _map = new SampleJoinDataIdMap();
      _fillInternalSequence();
      _dbclose();
    } catch(SQLException exc) {
      throw new DAException(DAResource.SELECT_FAILED, exc);
    } finally {
      _setBusy(false);
    }
    _managerSupport.fireSelectComplete("select(String)");
  }
  protected DataAccessObject newElement() { return new SampleJoinDataId();}
  public SampleJoinDataId element() { return (SampleJoinDataId)_cachedObject;}
  public static String getSQLBaseQuery() {
    return "select t1.key1" +
    ",t1.key2" +
    " from sample2 t1,sample t2"+
    " where t2.int_val = t1.key1";
  }
  public void _executeAction( String _methodName, Object[] _params ) throws Exception {
    if (_methodName.equals("prepareOpen(..)")) {
      prepareOpen((String)_params[0],(Object[])_params[1],(int[])_params[2]);
    }
    else super._executeAction(_methodName,_params);
  }

}
