import java.sql.*;
import java.util.*;
import JProjects.eab.data.*;
import java.io.Serializable;


class SampleManager extends DAManager {

  //----------------------------------------------------------------
  // Constructors
  //----------------------------------------------------------------
  public SampleManager() {}
  public SampleManager(DatastoreJDBC aDatastore) {
    super(aDatastore);
  }
  public DatastoreJDBC getCurrentDatastore() {
    if (getObjectsDatastore() != null) return getObjectsDatastore();
    else if (Sample.getDefaultDatastore() != null) return Sample.getDefaultDatastore();
    else return DatastoreJDBC.getApplicationDatastore();
  }
//----------------------------------------------------------------
// Open cursor with parameters
//----------------------------------------------------------------
  public void prepareOpen(String sqlSuffix,Object[] vals,int[] types) throws DAException {
    if (sqlSuffix==null|| vals==null|| types==null)
      throw new IllegalArgumentException();
    if (vals.length != types.length)
      throw new IllegalArgumentException();
    if ( !_onBackground() && isAsynchronous() ) {
      Object[] params = {sqlSuffix,vals,types};
      _putOnBackgroundThread( "prepareOpen(..)", params );
      return ;
    }
    try{
      if (getCurrentDatastore() == null) throw new DAException(DAResource.NO_CONNECT_EXIST);
      if (isOpen()) close();
      _stmt = getCurrentDatastore().getConnection().prepareStatement(getSQLBaseQuery() + sqlSuffix);
      _setStmtOptions();
      for (int i=0;i<types.length;i++)
        _stmt.setObject(i+1,vals[i],types[i]);
      _rs = _stmt.executeQuery();
      _map = new SampleMap();
      _setOpen(true);
    } catch(SQLException exc) {
      throw new DAException(DAResource.PREP_OPEN_FAILED, exc);
    } finally { _setBusy(false); }
    _managerSupport.firePrepareOpenComplete("prepareOpen(..)");
  }
//----------------------------------------------------------------
// Open cursor
//----------------------------------------------------------------
  public void open(String sqlSuffix) throws DAException {
    if ( !_onBackground() && isAsynchronous() ) {
      Object[] params = {sqlSuffix};
      _putOnBackgroundThread( "open(String)", params );
      return ;
    }
    try {
    if (getCurrentDatastore() == null) throw new DAException(DAResource.NO_CONNECT_EXIST);
      if (isOpen()) close();
      if (sqlSuffix == null) sqlSuffix = "";
      _stmt = getCurrentDatastore().getConnection().prepareStatement(getSQLBaseQuery() + sqlSuffix);
      _setStmtOptions();
      _rs = _stmt.executeQuery();
      _map = new SampleMap();
      _setOpen(true);
    } catch(SQLException exc) {
      throw new DAException(DAResource.OPEN_FAILED, exc);
    } finally {
      _setBusy(false);
    }
    _managerSupport.fireOpenComplete("open(String)");
  }
  //----------------------------------------------------------------
  // Select rows and fill internal sequence.
  //----------------------------------------------------------------
  public void select(String sqlSuffix) throws DAException {
    if ( !_onBackground() && isAsynchronous() ) {
      Object[] params = {sqlSuffix};
      _putOnBackgroundThread( "select(String)", params );
      return ;
    }
    try {
    if (getCurrentDatastore() == null) throw new DAException(DAResource.NO_CONNECT_EXIST);
      if (isOpen()) close();
      if (sqlSuffix == null) sqlSuffix = "";
      _stmt = getCurrentDatastore().getConnection().prepareStatement(getSQLBaseQuery() + sqlSuffix);
      _setStmtOptions();
      _rs = _stmt.executeQuery();
      _map = new SampleMap();
      _fillInternalSequence();
      _dbclose();
    } catch(SQLException exc) {
      throw new DAException(DAResource.SELECT_FAILED, exc);
    } finally {
      _setBusy(false);
    }
    _managerSupport.fireSelectComplete("select(String)");
  }
  protected DataAccessObject newElement() { return new Sample();}
  public Sample element() { return (Sample)_cachedObject;}
  public static String getSQLBaseQuery() {
    return "select t1.int_val" +
    ",t1.float_val" +
    ",t1.char_val" +
    ",t1.date_val" +
    " from sample t1";
  }
  public void _executeAction( String _methodName, Object[] _params ) throws Exception {
    if ( _methodName.equals("deleteFetched()") ) {
      deleteFetched();
    }
    else if ( _methodName.equals("updateFetched()") ) {
      updateFetched();
    }
    else if (_methodName.equals("prepareOpen(..)")) {
      prepareOpen((String)_params[0],(Object[])_params[1],(int[])_params[2]);
    }
    else super._executeAction(_methodName,_params);
  }
  public void deleteFetched() throws DAException {
    if ( !_onBackground() && isAsynchronous() ) {
      _putOnBackgroundThread( "deleteFetched()", null );
      return ;
    }
    try {
      if (!isOpen()) throw new DAException(DAResource.CURSOR_NOT_OPEN);
      if (!isFetched()) throw new DAException(DAResource.ROW_NOT_FETCHED);
      String stmStr = "delete from sample where current of " + _rs.getCursorName();
      PreparedStatement deleteStmt = getCurrentDatastore().getConnection().prepareStatement( stmStr );
      deleteStmt.executeUpdate();
      deleteStmt.close();
    } catch(SQLException exc) {
      throw new DAException(DAResource.ERROR_IN_METHOD,"deleteFetched()", exc);
    } finally {
      _setBusy(false);
    }
    _setFetched(false);
    element().setModified(false);
    _managerSupport.fireDeleteFetchedComplete();
  }
  public void updateFetched() throws DAException {
    if ( !_onBackground() && isAsynchronous() ) {
      _putOnBackgroundThread( "updateFetched()", null );
      return ;
    }
    Sample obj = element();
    try {
      if (!isOpen()) throw new DAException(DAResource.CURSOR_NOT_OPEN);
      if (!isFetched()) throw new DAException(DAResource.ROW_NOT_FETCHED);
      if (!obj.isModified()) throw new DAException(DAResource.NO_CHANGES_MADE);
      String stmStr = Sample.generator.genUpdateFetched(Sample.getQualifier(),
      obj.getDataId().modified, obj.modified) + _rs.getCursorName();
      PreparedStatement updateStmt = getCurrentDatastore().getConnection().prepareStatement( stmStr );
      int index = 1;
      if (obj.getDataId().modified[0]) {
  updateStmt.setInt(index++,obj.getIntVal());
      }
      if (obj.modified[0]) {
        if (obj.getFloatVal() == null) updateStmt.setNull(index++,Types.FLOAT);
        else updateStmt.setFloat(index++,obj.getFloatVal().floatValue());
      }
      if (obj.modified[1]) {
        if (obj.getCharVal() == null) updateStmt.setNull(index++,Types.CHAR);
        else updateStmt.setString(index++,obj.getCharVal());
      }
      if (obj.modified[2]) {
        if (obj.getDateVal() == null) updateStmt.setNull(index++,Types.DATE);
        else updateStmt.setDate(index++,obj.getDateVal());
      }
      updateStmt.executeUpdate();
      updateStmt.close();
    } catch(SQLException exc) {
      throw new DAException(DAResource.ERROR_IN_METHOD,"updateFetched()", exc);
    } finally {
      _setBusy(false);
    }
    obj.setModified(false);
    _managerSupport.fireUpdateFetchedComplete();
  }

}
